#!/usr/bin/env bash


#Parar en el primer error
# Mostrar comando
set -ex


##########################
# Definición del entorno
##########################
# Algunas variables para configurar el deploy
# Es lo único que hay que tocar en el script para ajustarlo

# Alias del host/s destino en el fichero ~/.ssh/config
# Host donde se hará deploy
if [ $BRANCH_NAME == 'master' ] ;then
    DEPLOY_HOSTALIAS="finect-pro-static1"
    DEPLOY_DIR="/srv/www/models.finect.com"
    DEVELOPER_HOSTALIAS="finect-pro-static1"
    DEVELOPER_DIR="/srv/www/developer.finect.com/docs"

elif [ $BRANCH_NAME == 'develop' ] ;then
    DEPLOY_HOSTALIAS="finect-stage-front1"
    DEPLOY_DIR="/srv/www/stage-models.finect.com"

    DEVELOPER_HOSTALIAS="finect-stage-front1"
    DEVELOPER_DIR="/srv/www/stage-developer.finect.com/docs"

fi


# Algunas variables para configurar el deploy
# Es lo único que hay que tocar en el script para ajustarlo

# Alias del host/s destino en el fichero ~/.ssh/config
# Host donde se hará deploy
DEPLOY_USER="www-data"

###########################
# Info
###########################

echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ""
echo ""
echo "Job Jenkins         ---> $JOB_NAME"
echo "Destino             ---> $DEPLOY_HOSTALIAS"
echo "Usuario en destino  ---> $DEPLOY_USER"
echo "Directorio          ---> $DEPLOY_DIR"
echo ""
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ""



##########################
# Gestión de Backups
##########################
backup(){
# Los comandos entre los EOSSH se ejecutan en la máquina remota con el usuario $DEPLOY_USER
ssh $DEPLOY_HOSTALIAS sudo -u $DEPLOY_USER -s bash <<- EOSSH

  set -e
  echo ">>> Preparación del entorno..."
  set -x
  rm -rf /tmp/$DEPLOY_DIR.tmp
  mkdir -p /tmp/$DEPLOY_DIR.tmp
  mkdir -p $DEPLOY_DIR/BACKUPS



  echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
  echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
  echo ">>> "
  echo ">>> "
  echo ">>> Backup de lo existente en $DEPLOY_DIR"
  echo ">>> "
  echo ">>> "
  echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
  echo ""

  set +e
  set -x
  cd $DEPLOY_DIR
  tar -zcvf $DEPLOY_DIR/BACKUPS/$JOB_NAME-`date +%Y%m%d_%H%M%S`.tgz .
  set +x
  set -e

  echo ""
  echo ">>> Borrado de backups antiguos"
  echo ""
  set -x
  find $DEPLOY_DIR/BACKUPS -mtime +10 -type f -exec rm -v {} \;
  set +x

EOSSH
}

##########################
# Transferencia del Sw
##########################
transferSW(){
# Los datos a transferir se vuelcan directamente al stream sin necesidad de crear ficheros disco: files > tar | ssh > tar > files
# La salida del tar es el Standard Out. Utilizando Pipes se puede transferir directamente sin necesidad de guardar una copia en disco
# En el extremo remoto se inyecta un comando al ssh que cambia al usuario correcto y expande el fichero recibido por el Standard Input.
local_tar_cmd="tar -zcvf -  ."
remote_tar_cmd="sudo -u $DEPLOY_USER -s tar -xzf - -C /tmp/$DEPLOY_DIR.tmp"

# Ejecución de la transferencia.
echo ""
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ">>> "
echo ">>> "
echo ">>> Transfiriendo proyecto..."
echo ">>> "
echo ">>> "
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ""
set -x
cd $WORKSPACE/models
$local_tar_cmd|ssh $DEPLOY_HOSTALIAS "$remote_tar_cmd"

set +x

# Actualización del proyecto
COMMIT=$(git log -n 1 --pretty=format:'%H')
ssh $DEPLOY_HOSTALIAS sudo -u $DEPLOY_USER -s bash <<- EOSSH
  set -e
  echo ""
  echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
  echo ">>> "
  echo ">>> "
  echo ">>> Actualizando proyecto.."
  echo ">>> "
  echo ">>> "
  echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
  echo ""
  set -x
  echo ">>> Borrando contenido actual.."
  rm -rf $DEPLOY_DIR/*
  ls
  echo ">>> Copiando al destino final.."
  cp -r  /tmp/$DEPLOY_DIR.tmp/* $DEPLOY_DIR

  echo ">>> Generando fichero de versión.."
cat <<EOF > ${DEPLOY_DIR}/version.json
{"build":"${BUILD_URL}", "commit":"${COMMIT}"}
EOF


  echo ">>> Elimando directorio temporal.."
  rm -r /tmp/$DEPLOY_DIR.tmp
  set +x

EOSSH
}


##########################
# Transferencia del fichero índice para SwaggerUI
##########################
transferSuaggerUI(){

local_tar_cmd="tar -zcvf -  swagger-ui.yaml"
remote_tar_cmd="sudo -u $DEPLOY_USER -s tar -xzf - -C $DEVELOPER_DIR"

# Ejecución de la transferencia.
echo ""
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ">>> "
echo ">>> "
echo ">>> Transfiriendo swagger-ui.yaml..."
echo ">>> "
echo ">>> "
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo ""
set -x
cd $WORKSPACE/models
$local_tar_cmd|ssh $DEVELOPER_HOSTALIAS "$remote_tar_cmd"


}


###################################################################################################
# PRINCIPAL
###################################################################################################
backup

transferSW

transferSuaggerUI
