'use strict';

const odata = require("odata-parser");

module.exports =  {
    parse : async (filter, mapFields=[]) => {
        const exp = odata.parse(filter);

        let result = { where: '', parameters: [] }
        await mapToMySql(exp.$filter, mapFields, result);

        return result;
    }
}  

async function mapToMySql(exp, mapFields, result, param={}) {

    if (exp.left) param = await mapToMySql(exp.left, mapFields, result, param);

    switch(exp.type.toLowerCase()) {
        case 'or':
            result.where += ' or ';
            break;
        case 'and':
            result.where += ' and ';
            break;
    }

    if (exp.right) param = await mapToMySql(exp.right, mapFields, result, param);
    if (param && param.name && param.value) result.parameters.push(param);

    switch(exp.type.toLowerCase()) {
        case 'property':
            result.where += (mapFields[exp.name] || exp.name).replace('/', '_');
            param.name = `${exp.name.toLowerCase()}_${Object.keys(result.parameters).length}`.replace('/', '_');

            return param;
        case 'literal':
            param.value = exp.value;
            return param;
        case 'eq':
            result.where += `=:${param.name}`;
            break;
        case 'lt':
            result.where += `<:${param.name}`;
            break;
        case 'gt':
            result.where += `>:${param.name}`;
            break;
        case 'le':
            result.where += `<=:${param.name}`;
            break;
        case 'ge':
            result.where += `>=:${param.name}`;
            break;
        case 'ne':
            result.where += `<>:${param.name}`;
            break;        
    }
}