'use strict';

const fs = require('fs');
// const path = require('path');
const winston = require('winston');
require('winston-daily-rotate-file');

// pre-checks
let valid = process.env.LOGS_PATH && process.env.LOGS_FILE;
if (!valid) throw new Error('Missing required environment variables');

if (!fs.existsSync(process.env.LOGS_PATH)){
  fs.mkdirSync(process.env.LOGS_PATH);
}

let logger = new (winston.Logger)({
  transports: [
    new (winston.transports.DailyRotateFile)({
      //handleExceptions: true,
      //exitOnError: false,
      level: process.env.LOGS_LEVEL || 'warn',       
      dirname: process.env.LOGS_PATH,
      filename: process.env.LOGS_FILE + '-%DATE%.log',
      datePattern: 'YYYY-MM-DD'
    })
  ]});

logger.add(winston.transports.Console, {
  colorize: true,
  timestamp: true,
  //handleExceptions: true,
  level: process.env.LOGS_LEVEL || 'info'
});

// Override the built-in console methods with winston hooks

/* eslint-disable node/no-unsupported-features */
/* eslint-disable no-console */
console.log = (...args) => logger.info.call(logger, ...args);
console.info = (...args) => logger.info.call(logger, ...args);
console.warn = (...args) => logger.warn.call(logger, ...args);
console.error = (...args) => logger.error.call(logger, ...args);
console.debug = (...args) => logger.debug.call(logger, ...args);
/* eslint-enable node/no-unsupported-features */
/* eslint-enable no-console */


module.exports = logger;
