'use strict';

require('dotenv').config();

const cluster = require("cluster");
const logger = require('@finect/logger');


module.exports.run = (config, listened) => {
	global.__basedir = config.appRoot; 
	global.__modelsdir = config.modelsdir;

	if (process.env.NODE_ENV === 'production' && cluster.isMaster) {
		const Master = require("./master");
		const master = new Master(cluster);
		const cpuCount = require("os").cpus().length;
	
		for (let i = 0; i < cpuCount; i++) {
			master.createWorker();
		}
	
		cluster.on("exit", (worker) => {
			logger.info("worker " + worker.id + "  died");
			master.spawnWorker(process.env.RESPAWN_WORKER_TIME_MS || 500);
		});
	} else {
		const Workers = require("./workers");
		const workers = new Workers(config);
		
		workers.run(listened);
		logger.info("worker running!");
	}
}
