'use strict';

describe('map', () => {
    let sqlmap;

    beforeEach(() => {
        sqlmap = require('../sqlmap');
    });

    it('nothing to do on simple array', () => {

        let arr = [
            {
                id: 1,
                name: 'Boston Celtics',
            },
            {
                id: 2,
                name: 'Los Angeles Lakers',
            }
        ];

        let result = sqlmap.mapDynamic(arr);
        console.assert(result.length === 2);

        JSON.stringify(result, null, 2);
    });

    it('mapping simple associations', () => {

        let arr = [
            {
                id: 1,
                name: 'Boston Celtics',
                coach_id: 1,
                coach_name: 'Doc Rivers',
            },
            {
                id: 2,
                name: 'Los Angeles Lakers',
                coach_id: 2,
                coach_name: 'Phil Jackson',
            }
        ];

        let result = sqlmap.mapDynamic(arr);
        console.assert(result.length === 2);
    });

    it('mapping simple collection', () => {

        let arr = [
            {
                id: 1,
                name: 'Boston Celtics',
                players__id: 2,
                players__name: 'Kevin Garnett',
                players__gamesPlayed: 71,
                players__points: 1337,
            },
            {
                id: 1,
                name: 'Boston Celtics',
                players__id: 1,
                players__name: 'Paul Pierce',
                players__gamesPlayed: 80,
                players__points: 1570
            },
            {
                id: 2,
                name: 'Los Angeles Lakers',
                players__id: 3,
                players__name: 'Kobe Bryant',
                players__gamesPlayed: 82,
                players__points: 2323
            },
            {
                id: 2,
                name: 'Los Angeles Lakers',
                players__id: 4,
                players__name: 'Pau Gasol',
                players__gamesPlayed: 66,
                players__points: 1246,
            }
        ];

        let result = sqlmap.mapDynamic(arr);

        console.assert(result.length === 2);
        console.assert(result[0].players.length === 2);
        console.assert(result[1].players.length === 2);

        
        console.log(JSON.stringify(result, null, 2));

    });

    it('mapping multiple associations', () => {

        let arr = [
            {
                id: 1,
                name: 'Boston Celtics',
                coach_id: 1,
                coach_name: 'Doc Rivers',
                subcoach_id: 2,
                subcoach_name: 'Phil Jackson',
            },
            {
                id: 2,
                name: 'Los Angeles Lakers',
                coach_id: 2,
                coach_name: 'Phil Jackson',
                subcoach_id: 2,
                subcoach_name: 'Doc Rivers',
            }
        ];

        let result = sqlmap.mapDynamic(arr);

        console.assert(result.length === 2);
        console.assert(result[0].hasOwnProperty('coach'));
        console.assert(result[0].hasOwnProperty('subcoach'));
        console.assert(result[1].hasOwnProperty('coach'));
        console.assert(result[1].hasOwnProperty('subcoach'));
    });    

    it('mapping multiple collections', () => {

        let arr = [
            {
                id: 1,
                name: 'Boston Celtics',
                coach_id: 1,
                coach_name: 'Doc Rivers',
                players__id: 2,
                players__name: 'Kevin Garnett',
                players__gamesPlayed: 71,
                players__points: 1337,

                sicks__id: 2,
                sicks__name: 'Aaaaa Bbbbb',
                sicks__gamesPlayed: 71,
                sicks__points: 1337,
                sicks__history__id: 1,
                sicks__history__description: 'pppppppppppp',
                sicks__cause_description: 'pppppppppppp',
                sicks__cause_date: 'date'
            },
            {
                id: 1,
                name: 'Boston Celtics',
                coach_id: 1,
                coach_name: 'Doc Rivers',
                players__id: 1,
                players__name: 'Paul Pierce',
                players__gamesPlayed: 80,
                players__points: 1570,

                sicks__id: 1,
                sicks__name: 'Ccccc Ccccc',
                sicks__gamesPlayed: 80,
                sicks__points: 1570,
                sicks__history__id: null,
                sicks__history__description: null,
                sicks__cause_description: 'vvvvvvvvvvvv',
                sicks__cause_date: 'date'
            },            
            {
                id: 2,
                name: 'Los Angeles Lakers',
                coach_id: 2,
                coach_name: 'Phil Jackson',
                players__id: 4,
                players__name: 'Pau Gasol',
                players__gamesPlayed: 66,
                players__points: 1246,

                sicks__id: 2,
                sicks__name: 'Aaaaa Bbbbb',
                sicks__gamesPlayed: 71,
                sicks__points: 1337,
                sicks__cause_description: 'pppppppppppp',
                sicks__cause_date: 'date'
            },            
            {
                id: 2,
                name: 'Los Angeles Lakers',
                coach_id: 2,
                coach_name: 'Phil Jackson',
                players__id: 3,
                players__name: 'Kobe Bryant',
                players__gamesPlayed: 82,
                players__points: 2323,

                sicks__id: 1,
                sicks__name: 'Ccccc Ccccc',
                sicks__gamesPlayed: 80,
                sicks__points: 1570,
                sicks__cause_description: 'vvvvvvvvvvvv',
                sicks__cause_date: 'date'
            }
        ];

        let result = sqlmap.mapDynamic(arr);

        console.assert(result.length === 2);
        console.log(JSON.stringify(result, null, 2));
    });      
    
    it('map using schema', () => {
        const schema = {
            "properties": {
                "code": {
                    "type": "number",
                    "description": "HTTP status code"
                },
                "internalCode": {
                    "type": "string",
                    "description": "Internal error code"
                },
                "message": {
                    "type": "string",
                    "description": "Error message (retorned only on develpment environment)"
                }
            },
            "required": [
                "code"
            ]
        };

        const arr = [
            {
                "code": 200,
                "internalCode": "-3000"
            }
        ]

        let result = sqlmap.mapDynamic(arr, schema);

        console.assert(result.length === 1);
        console.log(JSON.stringify(result, null, 2));
    });

    it('map using partial schema', () => {
        const schema = {
            "properties": {
                "text": {
                    "type": "string"
                },
                "metadata": {
                    "type": "array",
                    "items": {
                        "properties": {
                            "key": {
                                "type": "string"
                            },
                            "value": {
                                "type": "string"
                            }
                        },
                        "required": [
                            "key",
                            "value"
                        ]
                    }
                }
            },
            "required": [
                "text"
            ]
        };

        const arr = [
            {
                "key": "voluptate enim tempor",
                "value": "Ut incididunt Lorem"
            }
        ];

        let result = sqlmap.mapDynamic(arr, schema, "items");

        console.assert(result.length === 1);
        console.log(JSON.stringify(result, null, 2));
    });  
    
    it.only('allow special properties', () => {

        let arr = [
            {
                id: 1,
                $test: 'v1',
                name: 'Boston Celtics',
                coach_id: 1,
                coach_name: 'Doc Rivers',
                coach_$test: 'value 1',
                subcoach_id: 2,
                subcoach_name: 'Phil Jackson',
            },
            {
                id: 2,
                $test: 'v2',
                name: 'Los Angeles Lakers',
                coach_id: 2,
                coach_name: 'Phil Jackson',
                coach_$test: 'value 2',
                subcoach_id: 2,
                subcoach_name: 'Doc Rivers',
            }
        ];

        let result = sqlmap.mapDynamic(arr);

        console.assert(result.length === 2);
        console.assert(result[0].coach.hasOwnProperty('v1'));
        console.assert(result[1].coach.hasOwnProperty('v2'));
    });        
});
