'use strict';

const SwaggerParser = require('swagger-parser');

const _ = require('underscore');
const fs = require('fs');
const path = require('path');
const newman = require('newman');

const Endpoints = require('./collection-endpoints');

module.exports = (swagger, options, onfinished) => {

    const absSwaggerFile = path.resolve(swagger);
    const absDataFile = options.run && typeof options.run === 'string' ? path.resolve(options.run) : null;

    process.chdir(__dirname); 

    SwaggerParser.validate(absSwaggerFile)
        .then(async function(api) {
            if (absDataFile)
            {
                const globalData = require(absDataFile);
                globalData.forEach(data => {
                    Object.keys(data).forEach(key => { 
                        options.global.push({ name: key, value: data[key] })
                    });
                });
            }

            let endpoints = new Endpoints(api, options.global, options.tokenUrl);
            _.each(api.paths, (path, key) => {

                endpoints.parse(path, key);
            });   
            
            console.log('\nTesting endpoints definition\n');

            let collection = await endpoints.export((url) => {
                console.log('→ ' + url);
            });

            if (options.save)
            {
                let fileCollection = absSwaggerFile.substr(0, absSwaggerFile.lastIndexOf(".")) + '-postman.json';
                fs.writeFileSync(fileCollection, JSON.stringify(collection, null, 2));
                
                console.log('\nPostman collection exported to: ' + fileCollection);
            }

            if (options.run)
            {
                const newmanOptions = {
                    collection: collection,
                    reporters: 'cli'
                };

                if (absDataFile) newmanOptions.iterationData = absDataFile;

                console.log('\nOption --run detected. Using newman to run collection.');
                newman.run(newmanOptions, (err, summary) => {
                    if (err) { throw err; }
                    if (summary.run.failures.length > 0)
                    {
                        console.log(summary.run.failures.length + ' assertions failed.');
                        if (onfinished) onfinished(100);
                    }

                    console.log('collection run complete!');
                    if (onfinished) onfinished(0);
                });                
            }
        })
        .catch(function(err) {
            console.error('Ohnoes! :-( ' + err.message);
            if (onfinished) onfinished(100);
        });
};